﻿Rep2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
Rep2:RegisterDB("Rep2DB")
Rep2:RegisterDefaults("profile", {
	frame = 1,
} )

local factionVars = {}

local cwhite, cyellow, corange, cresume = "|cffffffff","|cffffff78","|cffff7831","|r"
local fmt = string.format
local getfactioninfo = GetFactionInfo
local math_abs = math.abs
local math_ceil = math.ceil

local standingmax = 8
local standingmin = 1

local options = {
	type = "group",
	desc = "Displays reputation gains.",
	args = {
		set = {
			type = "text",
			name = "set",
			desc = "Set the output chatframe.",
			get = function()
				return Rep2.db.profile.frame
			end,
			set = "SetChatFrame",
			usage = "ChatFrame#",
		},
	},
}

function Rep2:OnInitialize()
	self:RegisterChatCommand( {"/rep2", "/r2"}, options)
end

function Rep2:OnEnable()
	self:RegisterEvent("PLAYER_ALIVE", "ScanFaction")
	self:ScanFaction()
end

function Rep2:SetChatFrame(frameNum)
	frameNum = tonumber(frameNum)

	if frameNum then
		local framename = "ChatFrame" .. frameNum
		local chatframe = getglobal(framename)

		if chatframe then
			self:Print("Now using " .. framename)
			chatframe:AddMessage(cyellow .. "Rep2: " .. cwhite .. "Using this chat frame for reputation messages now!")

			self.db.profile.frame = frameNum;
		else
			self:Print(framename .. " is not a valid ChatFrame!")
		end
	end
end

function Rep2:ScanFaction()
	for i = 1, GetNumFactions() do
		local name, _, standingID, _, _, barValue = getfactioninfo(i)

		factionVars[name] = {}
		factionVars[name].Value = barValue
		factionVars[name].Standing = standingID
	end

	self:RegisterEvent("UPDATE_FACTION", "Report")
	self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE", "Report")
end

function Rep2:Report()
	local chatframe = getglobal("ChatFrame" .. self.db.profile.frame)

	for factionIndex = 1, GetNumFactions() do
		local name, _, standingID, barMin, barMax, barValue, _, _, isHeader = getfactioninfo(factionIndex)

		if (not isHeader) and factionVars[name] then
			local diff = barValue - factionVars[name].Value

			if diff ~= 0 then
				if standingID ~= factionVars[name].Standing then
					local newfaction = _G["FACTION_STANDING_LABEL" .. standingID]
					local newstandingtext =
						"New standing with " .. cyellow .. name .. cresume ..
						" is " .. cyellow .. newfaction .. cresume .. "!"

					chatframe:AddMessage(newstandingtext)
				end

				local remaining, nextstanding, plusminus
				if diff > 0 then
					plusminus = "+"
					remaining = barMax - barValue
					if standingID < standingmax then
						nextstanding = _G["FACTION_STANDING_LABEL" .. standingID + 1]
					else
						nextstanding = "End of " .. _G["FACTION_STANDING_LABEL" .. standingmax]
					end
				else
					plusminus = "-"
					remaining = barValue - barMin
					if standingID > standingmin then
						nextstanding = _G["FACTION_STANDING_LABEL" .. standingID - 1]
					else
						nextstanding = "Beginning of " .. _G["FACTION_STANDING_LABEL" .. standingmin]
					end
				end

				local change = math_abs(barValue - factionVars[name].Value)
				local repetitions = math_ceil(remaining / change)

				local newvaluetext = fmt("%s%s%d%s %s%s, %s%d%s more to %s%s%s (%s%d%s Repetitions).", 
				corange,plusminus,change,cyellow,name,cresume,corange,remaining,cresume,
				cyellow,nextstanding,cresume,corange,repetitions,cresume )

				chatframe:AddMessage(newvaluetext)

				factionVars[name].Value = barValue
				factionVars[name].Standing = standingID
			end
		end
	end
end
